// $Id: TaskBarAttributes.cpp,v 1.24 2011/11/11 13:29:09 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file TaskBarAttributes.cc
\brief taskbar attributes
\author Hugo Pereira
\version $Revision: 1.24 $
\date $Date: 2011/11/11 13:29:09 $
*/

#include "TaskBarAttributes.h"
#include "XmlDef.h"
#include "Str.h"

//_____________________________________
unsigned int& TaskBarAttributes::_idCount( void )
{
    static unsigned int counter = 0;
    return counter;
}

//_____________________________________
TaskBarAttributes::ModeNameMap& TaskBarAttributes::_modeNames( void )
{
    static TaskBarAttributes::ModeNameMap names;
    return names;
}

//________________________________________________________________________
TaskBarAttributes::LayoutNameMap& TaskBarAttributes::_layoutNames( void )
{
    static TaskBarAttributes::LayoutNameMap names;
    return names;
}

//________________________________________________________________________
TaskBarAttributes::PositionNameMap& TaskBarAttributes::_positionNames( void )
{
    static TaskBarAttributes::PositionNameMap names;
    return names;
}

//________________________________________________________________________
void TaskBarAttributes::_initializeModeNames( void )
{
    Debug::Throw( "TaskBarAttributes::_initializeModeNames.\n" );
    assert( _modeNames().empty() );
    _modeNames().insert( std::make_pair( FIXED, "fixed" ) );
    _modeNames().insert( std::make_pair( MINIMUM, "minimum" ) );
    _modeNames().insert( std::make_pair( OSX, "parabolic" ) );
}

//________________________________________________________________________
void TaskBarAttributes::_initializeLayoutNames( void )
{
    Debug::Throw( "TaskBarAttributes::_initializeLayoutNames.\n" );
    assert( _layoutNames().empty() );
    _layoutNames().insert( std::make_pair( HORIZONTAL_TOP, "horizontal top" ) );
    _layoutNames().insert( std::make_pair( HORIZONTAL_CENTER, "horizontal centered" ) );
    _layoutNames().insert( std::make_pair( HORIZONTAL_BOTTOM, "horizontal bottom" ) );

    _layoutNames().insert( std::make_pair( VERTICAL_LEFT, "vertical left" ) );
    _layoutNames().insert( std::make_pair( VERTICAL_CENTER, "vertical centered" ) );
    _layoutNames().insert( std::make_pair( VERTICAL_RIGHT, "vertical right" ) );

    _layoutNames().insert( std::make_pair( BOX, "box" ) );
}

//________________________________________________________________________
void TaskBarAttributes::_initializePositionNames( void )
{
    Debug::Throw( "TaskBarAttributes::_initializePositionNames.\n" );
    _positionNames().insert( std::make_pair( ANY_POSITION, "any" ) );
    _positionNames().insert( std::make_pair( TOP_LEFT_POSITION, "top left" ) );
    _positionNames().insert( std::make_pair( TOP_CENTER_POSITION, "top center" ) );
    _positionNames().insert( std::make_pair( TOP_RIGHT_POSITION, "top right" ) );

    _positionNames().insert( std::make_pair( BOTTOM_LEFT_POSITION, "bottom left" ) );
    _positionNames().insert( std::make_pair( BOTTOM_CENTER_POSITION, "bottom center" ) );
    _positionNames().insert( std::make_pair( BOTTOM_RIGHT_POSITION, "bottom right" ) );

    _positionNames().insert( std::make_pair( LEFT_CENTER_POSITION, "center left" ) );
    _positionNames().insert( std::make_pair( RIGHT_CENTER_POSITION, "center right" ) );

}

//____________________________________________________________________________________
QString TaskBarAttributes::positionName( const unsigned int& position )
{
    Debug::Throw() << "TaskBarAttributes::positionName - " << position << endl;
    PositionNameMap::const_iterator iter( positionNames().find( position ) );
    assert( iter != positionNames().end() );
    return iter->second;
}

//___________________________________________
TaskBarAttributes::TaskBarAttributes( const QDomElement& element ):
    Counter( "TaskBarAttributes" ),
    id_( _idCount()++ ),
    name_( "generic taskbar" ),
    layout_( HORIZONTAL_CENTER ),
    mode_( OSX ),
    nColumns_( 2 ),
    positionMode_( ANY_POSITION ),
    position_( 0, 0 ),
    zoomDelay_( 1 ),
    flags_(0),
    scale_( 5 ),
    needTaskUpdate_( false )
{
    Debug::Throw( "TaskBarAttributes::TaskBarAttributes.\n" );

    // parse attributes
    QDomNamedNodeMap attributes( element.attributes() );
    for( unsigned int i=0; i<attributes.length(); i++ )
    {
        QDomAttr attribute( attributes.item( i ).toAttr() );
        if( attribute.isNull() ) continue;
        QString name( attribute.name() );
        QString value( attribute.value() );

        if( name == XML::NAME ) setName( value );
        else if( name == XML::LAYOUT ) setLayout( value );
        else if( name == XML::MODE ) setMode( value );
        else if( name == XML::COLUMNS ) setNumColumns( value.toInt() );
        else if( name == XML::SCALE ) setScale( value.toDouble() );
        else if( name == XML::POSITION ) setPositionMode( value );
        else if( name == XML::X ) setX( value.toInt() );
        else if( name == XML::Y ) setY( value.toInt() );
        else if( name == XML::ZOOM_DELAY ) setZoomDelay( value.toInt() );
        else if( name == XML::FLAGS ) setFlags( value.toInt() );
        else Debug::Throw(0) << "TaskBarAttributes::TaskBarAttributes - unrecognized attribute " << name << endl;
    }

    // parse children elements
    for(QDomNode child_node = element.firstChild(); !child_node.isNull(); child_node = child_node.nextSibling() )
    {
        QDomElement child_element = child_node.toElement();
        if( child_element.isNull() ) continue;

        QString tag_name( child_element.tagName() );
        if( tag_name == XML::TASK ) addTaskAttributes( TaskAttributes( child_element ) );
        else Debug::Throw(0) << "TaskBarAttributes::TaskBarAttributes - unrecognized child " << tag_name << endl;
    }

}

//____________________________________________________________________________________
QDomElement TaskBarAttributes::domElement( QDomDocument& parent ) const
{
    Debug::Throw( "TaskBarAttributes::DomElement.\n" );

    QDomElement out( parent.createElement( XML::TASKBAR ) );
    if( !name().isEmpty() ) out.setAttribute( XML::NAME, name() );
    out.setAttribute( XML::LAYOUT, layoutName() );
    out.setAttribute( XML::MODE, modeName() );
    out.setAttribute( XML::COLUMNS, Str().assign<unsigned int>( numColumns() ) );
    out.setAttribute( XML::SCALE, Str().assign<double>( scale() ) );
    out.setAttribute( XML::POSITION, positionName() );
    out.setAttribute( XML::X, Str().assign<int>(position_.x()) );
    out.setAttribute( XML::Y, Str().assign<int>(position_.y()) );
    out.setAttribute( XML::ZOOM_DELAY, Str().assign<unsigned int>( zoomDelay() ) );
    out.setAttribute( XML::FLAGS, Str().assign<unsigned int>( flags() ) );

    // tasks
    for( TaskAttributes::List::const_iterator  iter = taskList_.begin(); iter != taskList_.end(); ++iter )
        out.appendChild( iter->domElement( parent ) );
    return out;

}

//____________________________________________________________________________________
bool TaskBarAttributes::needTaskUpdate( const TaskBarAttributes& attributes )
{
    Debug::Throw( "TaskBarAttributes::needTaskUpdate.\n" );

    if( needTaskUpdate_ )
    {
        needTaskUpdate_ = false;
        return true;
    }

    // check task list
    if( taskAttributesList() != attributes.taskAttributesList() ) return true;

    // check flags
    if( flag( EXPAND_SEPARATORS ) != attributes.flag( EXPAND_SEPARATORS ) ) return true;
    if( flag( SHOW_NAME ) != attributes.flag( SHOW_NAME ) ) return true;
    if( flag( ZOOM_ENABLED ) != attributes.flag( ZOOM_ENABLED ) ) return true;
    if( layout() != attributes.layout() ) return true;

    return false;
}

//____________________________________________________________________________________
void TaskBarAttributes::setLayout( const QString& layout )
{

    for( LayoutNameMap::const_iterator iter = layoutNames().begin(); iter != layoutNames().end(); ++iter )
    {
        if( layout == iter->second )
        {
            setLayout( iter->first );
            return;
        }
    }

    assert(0);

}

//____________________________________________________________________________________
void TaskBarAttributes::setMode( const QString& mode )
{

    for( ModeNameMap::const_iterator iter = modeNames().begin(); iter != modeNames().end(); ++iter )
    {
        if( mode == iter->second )
        {
            setMode( iter->first );
            return;
        }
    }

    assert(0);

}

//____________________________________________________________________________________
void TaskBarAttributes::setPositionMode( const QString& position )
{

    for( PositionNameMap::const_iterator iter = positionNames().begin(); iter != positionNames().end(); ++iter )
    {
        if( position == iter->second )
        {
            setPositionMode( iter->first );
            return;
        }
    }

}
