#ifndef _TaskBarAttributes_h_
#define _TaskBarAttributes_h_

// $Id: TaskBarAttributes.h,v 1.30 2011/11/11 13:29:09 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file TaskBarAttributes.h
\brief taskbar attributes
\author Hugo Pereira
\version $Revision: 1.30 $
\date $Date: 2011/11/11 13:29:09 $
*/

#include "Counter.h"
#include "Options.h"
#include "TaskAttributes.h"

#include <QtCore/QPoint>
#include <QtXml/QDomDocument>
#include <QtXml/QDomElement>

#include <cassert>
#include <vector>

//! single TaskAttributes attributesrmation
class TaskBarAttributes: public Counter
{

    public:

    //! task attributes list
    typedef std::vector<TaskBarAttributes> List;

    //! constructor
    TaskBarAttributes( const QDomElement& element = QDomElement() );

    //! dom element
    QDomElement domElement( QDomDocument& parent ) const;

    //! id
    const unsigned int& id() const
    { return id_; }

    //! equal to operator
    /*!
    only the options are compared. The TaskAttributesList
    must be compared independantly
    */
    bool operator == (const TaskBarAttributes& attributes ) const
    {
        return
            id() == attributes.id() &&
            name() == attributes.name() &&
            layout() == attributes.layout() &&
            mode() == attributes.mode() &&
            numColumns() == attributes.numColumns() &&
            zoomDelay() == attributes.zoomDelay() &&
            positionMode() == attributes.positionMode() &&
            scale() == attributes.scale() &&
            flags() == attributes.flags();
    }

    //! return true if attributes are different up to the point when tasks are to be updated
    bool needTaskUpdate( const TaskBarAttributes& );

    //! name
    const QString& name( void ) const
    { return name_; }

    //! name
    void setName( const QString& value )
    { name_ = value; }

    //!@name layout
    //@{

    //! layout enum
    enum Layout
    {
        TOP_LEFT = 1<<0,
        CENTER = 1<<1,
        BOTTOM_RIGHT = 1<<2,
        VERTICAL = 1<<3,
        HORIZONTAL = 1<<4,
        BOX = 1<<5,

        VERTICAL_CENTER = VERTICAL | CENTER,
        VERTICAL_LEFT = VERTICAL | TOP_LEFT,
        VERTICAL_RIGHT = VERTICAL | BOTTOM_RIGHT,

        HORIZONTAL_CENTER = HORIZONTAL | CENTER,
        HORIZONTAL_TOP = HORIZONTAL | TOP_LEFT,
        HORIZONTAL_BOTTOM = HORIZONTAL | BOTTOM_RIGHT

    };

    //! Layout
    const Layout& layout() const
    { return layout_; }

    //! map layout to layout name
    typedef std::map< Layout, QString > LayoutNameMap;

    //! Layout
    QString layoutName() const
    { return layoutName( layout() ); }

    //! layout names
    static QString layoutName( const Layout& layout )
    {
        if( _layoutNames().empty() ) _initializeLayoutNames();
        LayoutNameMap::const_iterator iter( _layoutNames().find( layout ) );
        assert( iter != _layoutNames().end() );
        return iter->second;
    }

    //! layout names
    static const LayoutNameMap& layoutNames()
    {
        if( _layoutNames().empty() ) _initializeLayoutNames();
        return _layoutNames();
    }

    //! Layout
    void setLayout( const Layout& layout )
    { layout_ = layout; }

    //! Layout
    void setLayout( const QString& layout );

    //@}

    //!@name mode
    //@{

    //! zoom mode
    enum Mode
    {
        FIXED,
        MINIMUM,
        OSX

    };

    //! Mode
    const Mode& mode() const
    { return mode_; }

    //! Mode
    QString modeName() const
    { return modeName( mode() ); }

    //! mode names
    static QString modeName( const Mode& mode )
    {
        ModeNameMap::const_iterator iter( modeNames().find( mode ) );
        assert( iter != modeNames().end() );
        return iter->second;
    }

    //! map mode to mode name
    typedef std::map< Mode, QString > ModeNameMap;

    //! layout names
    static const ModeNameMap& modeNames()
    {

        if( _modeNames().empty() ) _initializeModeNames();
        return _modeNames();
    }

    //! Mode
    void setMode( const Mode& mode )
    { mode_ = mode; }

    //! Mode
    void setMode( const QString& mode );

    //@}

    //! number of columns
    const unsigned int& numColumns( void ) const
    { return nColumns_; }

    //! number of columns
    void setNumColumns( const unsigned int value )
    { nColumns_ = value; }

    //!@name positions
    //@{

    //! predefined positions
    enum { n_positions_ = 8 };
    enum Position
    {
        ANY_POSITION = 0,
        TOP_POSITION = 1<<0,
        BOTTOM_POSITION = 1<<1,
        LEFT_POSITION = 1<<2,
        RIGHT_POSITION = 1<<3,
        CENTER_POSITION = 1<<4,
        TOP_LEFT_POSITION      = TOP_POSITION | LEFT_POSITION,
        TOP_CENTER_POSITION    = TOP_POSITION | CENTER_POSITION,
        TOP_RIGHT_POSITION     = TOP_POSITION | RIGHT_POSITION,
        BOTTOM_LEFT_POSITION   = BOTTOM_POSITION | LEFT_POSITION,
        BOTTOM_CENTER_POSITION = BOTTOM_POSITION | CENTER_POSITION,
        BOTTOM_RIGHT_POSITION  = BOTTOM_POSITION | RIGHT_POSITION,
        LEFT_CENTER_POSITION   = CENTER_POSITION | LEFT_POSITION,
        RIGHT_CENTER_POSITION  = CENTER_POSITION | RIGHT_POSITION
    };

    //! map mode to mode name
    typedef std::map< unsigned int, QString > PositionNameMap;

    //! layout names
    static const PositionNameMap& positionNames()
    {
        if( _positionNames().empty() ) _initializePositionNames();
        return _positionNames();
    }

    //! Layout
    QString positionName() const
    { return positionName( positionMode() ); }

    //! layout names
    static QString positionName( const unsigned int& position );

    //! position mode
    const unsigned int& positionMode( void ) const
    { return positionMode_; }

    //! podiyion mode
    void setPositionMode( const QString& );

    //! podiyion mode
    void setPositionMode( const unsigned int& position_mode )
    { positionMode_ = position_mode; }

    //@}

    //! position
    QPoint position( void ) const
    { return position_; }

    //! position
    void setPosition( const QPoint& point )
    { position_ = point;  }

    //! position
    void setX( const int &x )
    { position_.setX(x); }

    //! position
    void setY( const int &y )
    { position_.setY(y); }

    //! zoom delay
    const unsigned int& zoomDelay( void ) const
    { return zoomDelay_; }

    //! zoom delay
    void setZoomDelay( const unsigned int& delay )
    { zoomDelay_ = delay; }

    //!@name flags
    //@{

    //! flags
    enum Flag
    {

        NONE = 0,
        SHOW_NAME = 1<<0,
        LOCK_POSITION = 1<<1,
        LOCK_BUTTONS = 1<<2,
        EXPAND_SEPARATORS = 1<<3,
        STAYS_ON_TOP = 1<<4,
        AUTO_HIDE = 1<<5,
        ROUND_CORNERS = 1<<6,
        STICKY = 1<<8,
        HIDE_FROM_TASKBAR = 1<<9,
        ZOOM_ENABLED = 1<<10
    };


    //! flags
    void setFlag( const Flag& flag, const bool& value = true )
    {
        if( value ) flags_ |= flag;
        else flags_ &= (~flag);
    }

    //! flags
    void setFlags( const unsigned int& flags )
    { flags_ = flags; }

    //! flags
    bool flag( const unsigned int& flags ) const
    { return flags_ &flags; }

    //! flags
    const unsigned int& flags( void ) const
    { return flags_; }

    //@}

    //! scale
    const double& scale( void ) const
    { return scale_; }

    //! scale
    void setScale( const double& value )
    { scale_ = value; }

    //! task attributes
    const TaskAttributes::List& taskAttributesList( void ) const
    { return taskList_; }

    //! task attributes
    void setTaskAttributesList( const TaskAttributes::List& task_list )
    {
        taskList_ = task_list;
        needTaskUpdate_ = true;
    }

    //! clear task attributes
    void clear( void )
    { taskList_.clear(); }

    //! task attributes
    void addTaskAttributes( const TaskAttributes& attributes )
    { taskList_.push_back( attributes ); }

    private:

    //! mode names
    static void _initializeModeNames( void );

    //! layout names
    static void _initializeLayoutNames( void );

    //! position names
    static void _initializePositionNames( void );

    //! id count
    static unsigned int& _idCount( void );

    //! map mode to name
    static ModeNameMap& _modeNames( void );

    //! map layout to name
    static LayoutNameMap& _layoutNames( void );

    //! map position mode to name
    static PositionNameMap& _positionNames( void );

    //! id
    unsigned int id_;

    //! name
    QString name_;

    //! layout
    Layout layout_;

    //! mode
    Mode mode_;

    //! number of columns
    unsigned int nColumns_;

    //!@name position
    //@{

    //! predefined position
    unsigned int positionMode_;

    //! position
    QPoint position_;

    //@}

    //! delay
    unsigned zoomDelay_;

    //! flags (bitwise or of Flag object)
    unsigned int flags_;

    //! zoom scale (OSX mode)
    double scale_;

    //! task attributes
    TaskAttributes::List taskList_;

    //! true when tasks needs to be updated
    bool needTaskUpdate_;

    //! dump to Xml
    friend QTextStream& operator << (QTextStream& out, const TaskBarAttributes& );

};

#endif
